package com.mmoragues.herois.data.database

import androidx.room.Room
import androidx.test.core.app.ApplicationProvider
import androidx.test.ext.junit.runners.AndroidJUnit4
import androidx.test.filters.SmallTest
import com.google.common.truth.Truth.assertThat
import com.mmoragues.herois.domain.model.Hero
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.runBlocking
import org.junit.After
import org.junit.Before
import org.junit.Test
import org.junit.runner.RunWith

/**
 * Test del LocalDataSource
 * Es crea una BBDD Room en memòria i es popula amb dades per fer les proves
 */
@RunWith(AndroidJUnit4::class)
@SmallTest
class LocalDataSourceImplTest {
    private lateinit var database: HeroDatabase

    @Before
    fun setup() {
        database = Room.inMemoryDatabaseBuilder(
            ApplicationProvider.getApplicationContext(),
            HeroDatabase::class.java
        )
            .allowMainThreadQueries()
            .build()
    }

    @After
    fun tearDown() {
        database.close()
    }

    //Provem que es pot insertar correctament els herois i es recuperen correctament
    @Test
    fun insertAndGetAllHeroes() = runBlocking {
        val heroes: MutableList<Hero> = mutableListOf()
        ('a'..'c').forEachIndexed { index, c ->
            heroes.add(Hero(id = index, name = c.toString()))
        }

        val localDataSourceImpl = LocalDataSourceImpl(database)

        localDataSourceImpl.insertHeroes(heroes)

        val heroesRecuperats = localDataSourceImpl.getAllHeroes().first()
        assertThat(heroesRecuperats).isEqualTo(heroes)
        heroesRecuperats.forEach {
            System.out.println(it.id)
        }
        System.out.println("Insert Heroes OK")
    }

    // Provem que es poden afegir nous herois correctament
    @Test
    fun updateWithNewHeroes() = runBlocking {
        val heroesInicial: MutableList<Hero> = mutableListOf()
        ('a'..'b').forEachIndexed { index, c ->
            heroesInicial.add(Hero(id = index, name = c.toString()))
        }

        val localDataSourceImpl = LocalDataSourceImpl(database)

        localDataSourceImpl.insertHeroes(heroesInicial)

        val heroesModificat: MutableList<Hero> = mutableListOf()
        ('a'..'c').forEachIndexed { index, c ->
            heroesModificat.add(Hero(id = index, name = c.toString()))
        }

        localDataSourceImpl.insertHeroes(heroesModificat)

        val heroesRecuperats = localDataSourceImpl.getAllHeroes().first()
        assertThat(heroesRecuperats).isEqualTo(heroesModificat)
        heroesRecuperats.forEach {
            System.out.println(it.id)
        }
        System.out.println("Update Heroes OK")
    }

    //Provem que a l'actualitzar els herois, es manté el camp isFavorite
    @Test
    fun keepFavoriteFieldWhenUpdate() = runBlocking {

        val heroesInicial: MutableList<Hero> = mutableListOf()
        ('a'..'b').forEachIndexed { index, c ->
            heroesInicial.add(Hero(id = index, name = c.toString(), isFavorite = true))
        }

        val localDataSourceImpl = LocalDataSourceImpl(database)

        localDataSourceImpl.insertHeroes(heroesInicial)

        val heroesModificat: MutableList<Hero> = mutableListOf()
        ('a'..'c').forEachIndexed { index, c ->
            heroesModificat.add(Hero(id = index, name = c.toString()))
        }

        localDataSourceImpl.insertHeroes(heroesModificat)
        val heroesRecuperats = localDataSourceImpl.getAllHeroes().first()
        assertThat(heroesRecuperats[0].isFavorite).isTrue()
        assertThat(heroesRecuperats[1].isFavorite).isTrue()
        assertThat(heroesRecuperats[2]).isEqualTo(heroesModificat[2])

        System.out.println("Manté el camp is Favorite - OK")
    }

    // Obtenim un heroi pel seu ID correctament
    @Test
    fun getHeroById() = runBlocking {

        val heroes: MutableList<Hero> = mutableListOf()
        ('a'..'b').forEachIndexed { index, c ->
            heroes.add(Hero(id = index, name = c.toString()))
        }

        val localDataSourceImpl = LocalDataSourceImpl(database)

        localDataSourceImpl.insertHeroes(heroes)

        val heroRecuperat = localDataSourceImpl.getHeroById(0).first()
        assertThat(heroRecuperat).isEqualTo(heroes[0])

        System.out.println("Get hero by id - OK")
    }

    // Canviem l'estat de isFavorite correctament
    @Test
    fun toggleIsFavorite() = runBlocking {

        val heroes: MutableList<Hero> = mutableListOf()
        ('a'..'b').forEachIndexed { index, c ->
            heroes.add(Hero(id = index, name = c.toString()))
        }

        val localDataSourceImpl = LocalDataSourceImpl(database)

        localDataSourceImpl.insertHeroes(heroes)

        localDataSourceImpl.toggleIsFavorite(0)
        var heroRecuperat = localDataSourceImpl.getHeroById(0).first()
        assertThat(heroRecuperat.isFavorite).isTrue()

        localDataSourceImpl.toggleIsFavorite(0)
        heroRecuperat = localDataSourceImpl.getHeroById(0).first()
        assertThat(heroRecuperat.isFavorite).isFalse()

        System.out.println("Canviar Is Favorite - OK")
    }

    //Obtenim el llistat de herois favorits
    @Test
    fun getFavoritesHeroes() = runBlocking {
        val heroesFavorites: MutableList<Hero> = mutableListOf()
        ('a'..'b').forEachIndexed { index, c ->
            heroesFavorites.add(Hero(id = index, name = c.toString(), isFavorite = true))
        }

        val heroesNoFavorites: MutableList<Hero> = mutableListOf()
        ('c'..'d').forEachIndexed { index, c ->
            heroesNoFavorites.add(Hero(id = index + 2, name = c.toString()))
        }

        val localDataSourceImpl = LocalDataSourceImpl(database)

        localDataSourceImpl.insertHeroes(heroesFavorites)
        localDataSourceImpl.insertHeroes(heroesNoFavorites)

        val heroesFavoritsRecuperats = localDataSourceImpl.getFavoritesHeroes().first()
        assertThat(heroesFavoritsRecuperats).isEqualTo(heroesFavorites)

        System.out.println("Get Favorites - OK")
    }

    //Obtenim un heroi aleatori
    @Test
    fun getRandomHero() = runBlocking {
        val heroes: MutableList<Hero> = mutableListOf()
        ('a'..'z').forEachIndexed { index, c ->
            heroes.add(Hero(id = index, name = c.toString()))
        }

        val localDataSourceImpl = LocalDataSourceImpl(database)

        localDataSourceImpl.insertHeroes(heroes)

        val randomHero = localDataSourceImpl.getRandomHero()

        assertThat(randomHero).isIn(heroes)

        System.out.println("Random Hero OK - Id Random Hero: ${randomHero.id}")

    }
}