package com.mmoragues.herois.ui.screens.endToEndTest

import android.content.Context
import androidx.compose.ui.test.*
import androidx.compose.ui.test.junit4.createAndroidComposeRule
import androidx.navigation.NavType
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.navigation.navArgument
import androidx.test.core.app.ApplicationProvider
import com.mmoragues.herois.MainActivity
import com.mmoragues.herois.R
import com.mmoragues.herois.common.TestConstants
import com.mmoragues.herois.common.TestConstants.LOADING
import com.mmoragues.herois.di.AppModule
import com.mmoragues.herois.ui.navigation.BottomMenuOption
import com.mmoragues.herois.ui.navigation.Screen
import com.mmoragues.herois.ui.screens.favoritesScreen.FavoritesScreen
import com.mmoragues.herois.ui.screens.heroDetailScreen.HeroDetailScreen
import com.mmoragues.herois.ui.screens.homeScreen.HomeScreen
import com.mmoragues.herois.ui.theme.HeroisTheme
import dagger.hilt.android.testing.HiltAndroidRule
import dagger.hilt.android.testing.HiltAndroidTest
import dagger.hilt.android.testing.UninstallModules
import org.junit.Before
import org.junit.Rule
import org.junit.Test

//Test extrem a extrem de l'aplicació
@HiltAndroidTest
@UninstallModules(AppModule::class)
class EndToEndTests {
    @get:Rule(order = 0)
    val hiltRule = HiltAndroidRule(this)

    @get:Rule(order = 1)
    val composeRule = createAndroidComposeRule<MainActivity>()

    @Before
    fun setUp() {
        hiltRule.inject()
        composeRule.setContent {
            val navController = rememberNavController()
            HeroisTheme {
                NavHost(
                    navController = navController,
                    startDestination = BottomMenuOption.Home.route
                )
                {
                    // Pantalla Home
                    composable(route = BottomMenuOption.Home.route)
                    {
                        HomeScreen(navController = navController)
                    }

                    //Pantalla Favorits
                    composable(
                        route = BottomMenuOption.Favorite.route
                    ) {
                        FavoritesScreen(navController = navController)

                    }

                    //Pantalla amb el detall de l'heroi
                    composable(
                        route = Screen.HeroDetailScreen.route + "/{heroId}",
                        arguments = listOf(
                            navArgument("heroId") {
                                type = NavType.IntType
                                defaultValue = 0
                                nullable = false
                            }
                        )
                    ) { entry ->
                        HeroDetailScreen(navController = navController)
                    }
                }
            }
        }
    }

    // Busquem un heroi i comprovem la seva fitxa de detall es vegi correctament
    @Test
    fun searchAHeroAndShowDetails() {
        //Busquem un heroi i anem a a la seva fitxa de detall
        composeRule.onNodeWithContentDescription("Search Icon").performClick()
        composeRule.onNodeWithTag(TestConstants.SEARCHTEXT).performTextInput("c")
        composeRule.onNodeWithTag("Card c").performClick()

        //Confirmem que estem a la fitxa de l'heroi que correspon
        composeRule.onNodeWithText("c").assertExists()

        //Confirmem que no es mostra la icona d'heroi random
        composeRule.onNodeWithContentDescription("Random Icon").assertDoesNotExist()

        //Es movem al final de la pantalla
        composeRule.onNodeWithTag("LazyColumn")
            .performScrollToNode(hasTestTag(TestConstants.WORKSECTION))

        //Comprovem que la seccion de Biografia es mostra i s'amaga corrrectament
        composeRule.onNodeWithTag(TestConstants.BIOGRAPHYSECTION).performClick()
        composeRule.onNodeWithTag(TestConstants.BIOGRAPHYSECTIONDETAIL, useUnmergedTree = true)
            .assertExists()
        composeRule.onNodeWithTag(TestConstants.BIOGRAPHYSECTION).performClick()
        composeRule.onNodeWithTag(TestConstants.BIOGRAPHYSECTIONDETAIL, useUnmergedTree = true)
            .assertDoesNotExist()

        //Comprovem que la seccion d'Aparença es mostra i s'amaga corrrectament
        composeRule.onNodeWithTag(TestConstants.APPEARANCESECTION).performClick()
        composeRule.onNodeWithTag(TestConstants.APPEARANCESECTIONDETAIL, useUnmergedTree = true)
            .assertExists()
        composeRule.onNodeWithTag(TestConstants.APPEARANCESECTION).performClick()
        composeRule.onNodeWithTag(TestConstants.APPEARANCESECTIONDETAIL, useUnmergedTree = true)
            .assertDoesNotExist()

        //Comprovem que la seccion de Connexions es mostra i s'amaga corrrectament
        composeRule.onNodeWithTag(TestConstants.CONNECTIONSECTION).performClick()
        composeRule.onNodeWithTag(TestConstants.CONNECTIONSECTIONDETAIL, useUnmergedTree = true)
            .assertExists()
        composeRule.onNodeWithTag(TestConstants.CONNECTIONSECTION).performClick()
        composeRule.onNodeWithTag(TestConstants.CONNECTIONSECTIONDETAIL, useUnmergedTree = true)
            .assertDoesNotExist()

        //Comprovem que la seccion de Treball es mostra i s'amaga corrrectament
        composeRule.onNodeWithTag(TestConstants.WORKSECTION).performClick()
        composeRule.onNodeWithTag(TestConstants.WORKSECTIONDETAIL, useUnmergedTree = true)
            .assertExists()
        composeRule.onNodeWithTag(TestConstants.WORKSECTION).performClick()
        composeRule.onNodeWithTag(TestConstants.WORKSECTIONDETAIL, useUnmergedTree = true)
            .assertDoesNotExist()

        //Tornem a la pàgina Home
        composeRule.onNodeWithContentDescription("Arrow Back").performClick()
    }

    //Comprovem que es pot marcar i desmarcar els herois com a Favorits des de la pantalla de Home
    @Test
    fun markAndUnmarkAHeroAsFavoriteFromHome() {

        val context = ApplicationProvider.getApplicationContext<Context>()
        val favoritesIcon = context.getString(R.string.option_favorites)
        val homeIcon = context.getString(R.string.option_home)

        //Marquem dos herois com a farorits a la pàgina de Home
        composeRule.onNodeWithTag("Card c").onChildren()
            .filterToOne(hasContentDescription("Favorite Icon")).performClick()
        composeRule.onNodeWithTag("Card d").onChildren()
            .filterToOne(hasContentDescription("Favorite Icon")).performClick()

        //Anem a la pàgina de farovits i confirmem que els dos herois estan a la llista
        composeRule.onNodeWithContentDescription(favoritesIcon).performClick()
        composeRule.onNodeWithTag("Card c").assertExists()
        composeRule.onNodeWithTag("Card d").assertExists()

        //Desmarquem un heroi i comprovem que ja no surt a la llista
        composeRule.onNodeWithTag("Card c").onChildren()
            .filterToOne(hasContentDescription("Favorite Icon")).performClick()
        composeRule.onNodeWithTag("Card c").assertDoesNotExist()

        //Tornem a la pàgina Home i desmarquem l'altre heroi
        composeRule.onNodeWithContentDescription(homeIcon).performClick()
        composeRule.onNodeWithTag("Card d").onChildren()
            .filterToOne(hasContentDescription("Favorite Icon")).performClick()

        //Tornem a la la pàgina de Favorits i confirmem que ja no surt l'heroi anterior
        composeRule.onNodeWithContentDescription(favoritesIcon).performClick()
        composeRule.onNodeWithTag("Card d").assertDoesNotExist()
    }

    //Provem que podem marcar i desmarcar els herois com a favorits des de la fitxa de detall de cada heroir
    @Test
    fun markAndUnmarkAHeroAsFavoriteFromDetail() {
        val context = ApplicationProvider.getApplicationContext<Context>()
        val favoritesIconMenu = context.getString(R.string.option_favorites)
        val homeIconMenu = context.getString(R.string.option_home)

        //Esperem a que es carreguin les dades
        composeRule.waitUntil {
            composeRule
                .onAllNodesWithTag(LOADING)
                .fetchSemanticsNodes().isEmpty()
        }

        //Anem al detall d'un heroi i el marquem com a favorit
        composeRule.onNodeWithTag("Card c").performClick()
        composeRule.onNodeWithContentDescription("Favorite Icon").performClick()

        //Tornem a Home i anem als favorits i confirmem que l'heroi està a la llista de favorits
        composeRule.onNodeWithContentDescription("Arrow Back").performClick()
        composeRule.onNodeWithContentDescription(favoritesIconMenu).performClick()
        composeRule.onNodeWithTag("Card c").assertExists()

        //Tornem a la pàgina Home i al detall de l'heori i el desmarquem com a Favorit
        composeRule.onNodeWithContentDescription(homeIconMenu).performClick()
        composeRule.onNodeWithTag("Card c").performClick()
        composeRule.onNodeWithContentDescription("Favorite Icon").performClick()

        //Tornem a Home i anem a Favorits i confirmem que ja no surt l'heroi
        composeRule.onNodeWithContentDescription("Arrow Back").performClick()
        composeRule.onNodeWithContentDescription(favoritesIconMenu).performClick()
        composeRule.onNodeWithTag("Card c").assertDoesNotExist()

    }
}