package com.mmoragues.herois.ui.screens.homeScreen

import android.content.Context
import androidx.compose.ui.test.*
import androidx.compose.ui.test.junit4.createAndroidComposeRule
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import androidx.test.core.app.ApplicationProvider
import com.mmoragues.herois.MainActivity
import com.mmoragues.herois.R.string.heroesList_noResult
import com.mmoragues.herois.common.TestConstants
import com.mmoragues.herois.common.TestConstants.LOADING
import com.mmoragues.herois.common.TestConstants.SEARCHTEXT
import com.mmoragues.herois.di.AppModule
import com.mmoragues.herois.ui.navigation.BottomMenuOption
import com.mmoragues.herois.ui.theme.HeroisTheme
import dagger.hilt.android.testing.HiltAndroidRule
import dagger.hilt.android.testing.HiltAndroidTest
import dagger.hilt.android.testing.UninstallModules
import org.junit.Before
import org.junit.Rule
import org.junit.Test

// Proves de la pantalla inicial de l'aplicació
@HiltAndroidTest
@UninstallModules(AppModule::class)
class HomeScreenTest{

    @get:Rule(order = 0)
    val hiltRule = HiltAndroidRule(this)

    @get:Rule(order = 1)
    val composeRule = createAndroidComposeRule<MainActivity>()

    @Before
    fun setUp(){
        hiltRule.inject()
        composeRule.setContent {
            val navController = rememberNavController()
            HeroisTheme {
                NavHost(
                    navController = navController,
                    startDestination = BottomMenuOption.Home.route
                ){
                    composable(route = BottomMenuOption.Home.route){
                            HomeScreen(navController = navController)
                    }
                }
            }
        }
    }


    /**
     * Obrim i tanquem el buscador
     */
    @Test
    fun searchText_OpenAndClose(){
        composeRule.onNodeWithTag(SEARCHTEXT).assertDoesNotExist()
        composeRule.onNodeWithContentDescription("Search Icon").performClick()
        composeRule.onNodeWithTag(SEARCHTEXT).assertIsDisplayed()
        composeRule.onNodeWithContentDescription("Close Icon").performClick()
        composeRule.onNodeWithTag(SEARCHTEXT).assertDoesNotExist()
    }

    /**
     * Filtrem per un heroi i comprovem que només apareix l'heroi filtrat i no els altres
     * Borrem el filtre i comprovem que surten tots els herois
     * Es comprova que el botó per esborrar el contingut apareix quan toca i desapareix quan no és necessari
     * Es comproven que surt el missatge correcte quan no es troba l'heroi que es filtra
     */
    @Test
    fun homeScreen_Filter(){

        // Obrim el buscador i busquem l'heroi "a"
        composeRule.onNodeWithTag(SEARCHTEXT).assertDoesNotExist()
        composeRule.onNodeWithContentDescription("Search Icon").performClick()
        composeRule.onNodeWithTag("Card a").assertExists()
        composeRule.onNodeWithTag("Card b").assertExists()
        composeRule.onNodeWithContentDescription("Delete Icon").assertDoesNotExist()
        composeRule.onNodeWithTag(SEARCHTEXT).performTextInput("a")
        composeRule.onNodeWithTag("Card a").assertExists()
        composeRule.onNodeWithTag("Card b").assertDoesNotExist()

        // Borrem la busqueda i confirem que tornen a sortir els altres herois
        composeRule.onNodeWithContentDescription("Delete Icon").assertExists()
        composeRule.onNodeWithContentDescription("Delete Icon").performClick()
        composeRule.onNodeWithTag("Card a").assertExists()
        composeRule.onNodeWithTag("Card b").assertExists()
        composeRule.onNodeWithContentDescription("Delete Icon").assertDoesNotExist()

        //Fem una búsqueda erronia i confirmem que surt el missatge corresponent
        val context = ApplicationProvider.getApplicationContext<Context>()
        val noResults = context.getString(heroesList_noResult)
        composeRule.onNodeWithTag(SEARCHTEXT).performTextInput("inventat")
        composeRule.onNodeWithText(noResults, true).assertExists()

        //Tanquem el buscador i confirmem que ja no surt el requadre de texte
        composeRule.onNodeWithContentDescription("Close Icon").performClick()
        composeRule.onNodeWithTag(SEARCHTEXT).assertDoesNotExist()
    }

}