package com.mmoragues.herois.common

import android.content.res.Resources
import android.util.Log
import com.mmoragues.herois.R
import kotlinx.coroutines.flow.*
import retrofit2.HttpException
import java.io.IOException

/**
 * Funció auxiliar que permet que l'aplicació pugui funcionar offline i actualitzar les dades des d'Internet
 * Primer agafa les dades de la BBDD i les emet indicant que està carregant més dades
 * A continuació, intenta obtenir les dades des d'Internet.
 * si les obté, les guarda a la BBDD i les emet indicant que ha tingut èxit
 * Si hi ha algun error, emet les dades de la BBDD sense actualitzar indicant que hi ha un error i quin error és
 */
inline fun <ResultType, RequestType> networkBoundResource(
    crossinline query: () -> Flow<ResultType>,
    crossinline fetch: suspend () -> RequestType,
    crossinline saveFetchResult: suspend (RequestType) -> Unit,
    crossinline shouldFetch: (ResultType) -> Boolean = { true }
) = flow {
    val data = query().first()

    val flow = if (shouldFetch(data)) {
        emit(Resource.Loading(data))

        try {
            saveFetchResult(fetch())
            query().map { Resource.Success(it) }
        } catch (e: HttpException) {
            query().map { Resource.Error(R.string.error_server, it) }
        } catch (e: IOException) {
            query().map { Resource.Error(R.string.error_internet, it) }
        }
        catch (e: Exception) {
            query().map { Resource.Error(R.string.error_unexpected, it) }
        }
    } else {
        query().map { Resource.Success(it) }
    }

    emitAll(flow)
}