package com.mmoragues.herois.ui.screens.favoritesScreen

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.mmoragues.herois.R
import com.mmoragues.herois.domain.model.Hero
import com.mmoragues.herois.domain.useCase.GetFavoritesHeroesUseCase
import com.mmoragues.herois.domain.useCase.ToggleFavoriteUseCase
import com.mmoragues.herois.ui.sharedComponents.searchBar.SearchWidgetState
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.launchIn
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * View Model de la pantalla Favorits
 */
@HiltViewModel
class FavoritesViewModel @Inject constructor(
    private val getFavoritesHeroesUseCase: GetFavoritesHeroesUseCase,
    private val toggleFavoriteUseCase: ToggleFavoriteUseCase
): ViewModel() {

    //Estat de widget per filtrar els herois (obert o tancat)
    var searchWidgetState by mutableStateOf(SearchWidgetState.CLOSED)
        private set

    //Valor del text a buscar
    var searchTextState by mutableStateOf("")
        private set

    //Llistat de Herois de la BBDD
    private var cachedHeroList = listOf<Hero>()

    //Llistat d'herois una vegada filtrats pel nom que surti al requadre de búsqueda
    var favoritesHeroes by mutableStateOf(listOf<Hero>())
        private set

    //Missatge que es mostra si no hi ha herois a la llista
    var noFavoritesHeroesMessage by mutableStateOf(-1)
        private set

    init {
        getFavoritesHeroes()
    }

    private fun getFavoritesHeroes(){
        getFavoritesHeroesUseCase().onEach { heroes ->
            cachedHeroList = heroes
            favoritesHeroes = cachedHeroList.filter { hero ->
                hero.name.contains(
                    searchTextState,
                    ignoreCase = true
                )
            }
            updateNoFavoritesHeroesMessage()
        }.launchIn(viewModelScope)
    }

    fun onEvent(event: FavoritesScreenEvent){
        when (event){
            is FavoritesScreenEvent.OnFavoriteClick -> {
                viewModelScope.launch {
                    toggleFavoriteUseCase(event.hero)
                }
            }
            is FavoritesScreenEvent.OnUpdateSearchWidgetState -> {
                searchWidgetState = event.newState
            }
            is FavoritesScreenEvent.OnUpdateSearchTextState -> {
                searchTextState = event.newValue
                favoritesHeroes = cachedHeroList.filter { hero ->
                    hero.name.contains(
                        event.newValue,
                        ignoreCase = true
                    )
                }
                updateNoFavoritesHeroesMessage()
            }
        }
    }

    private fun updateNoFavoritesHeroesMessage(){
        noFavoritesHeroesMessage = when {
            cachedHeroList.isEmpty() -> {
                R.string.heroesFavorites_empty
            }
            favoritesHeroes.isEmpty() -> {
                R.string.heroesList_noResult
            }
            else -> {
                -1
            }
        }
    }
}