package com.mmoragues.herois.ui.screens.gameScreen

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import androidx.lifecycle.viewmodel.compose.viewModel
import com.mmoragues.herois.R
import com.mmoragues.herois.domain.model.Hero
import com.mmoragues.herois.domain.useCase.GetRandomHeroUseCase
import com.mmoragues.herois.ui.screens.gameScreen.flipCard.CardFace
import com.mmoragues.herois.ui.screens.gameScreen.flipCard.CardPosition
import com.mmoragues.herois.ui.sharedComponents.powerStat.PowerName
import com.mmoragues.herois.ui.sharedComponents.powerStat.parsePowerValue
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * View Model de la pantalla de Joc
 */
@HiltViewModel
class GameViewModel @Inject constructor(
    private val getRandomHeroUseCase: GetRandomHeroUseCase
) : ViewModel() {

    var hasChosse = false

    var heroTopCard by mutableStateOf(Hero())
        private set
    var topCardIsMyCard by mutableStateOf(false)
        private set
    var heroBottomCard by mutableStateOf(Hero())
        private set
    var bottomCardIsMyCard by mutableStateOf(false)
        private set
    var topCardFace by mutableStateOf(CardFace.Front)
        private set
    var bottomCardFace by mutableStateOf(CardFace.Front)
        private set
    var titleId by mutableStateOf(R.string.game_title_chooseCard)
        private set

    var startBetAnimation by mutableStateOf(false)
        private set

    var chosenPower by mutableStateOf(PowerName.COMBAT)
        private set

    var chosenPowerValueTopCard by mutableStateOf(0)
        private set

    var chosenPowerValueBottomCard by mutableStateOf(0)
        private set

    var youHaveWon by mutableStateOf(false)
        private set

    var youHaveLost by mutableStateOf(false)
        private set

    var youHaveTied by mutableStateOf(false)
        private set

    init {
        getCards()
    }

    private fun getCards() {
        viewModelScope.launch {
            heroTopCard = getRandomHeroUseCase() //.first()
            heroBottomCard = getRandomHeroUseCase() //.first()
        }
    }

    fun onEvent(event: GameScreenEvent) {
        when (event) {
            is GameScreenEvent.OnTopCardClick -> {
                if (!hasChosse) {
                    hasChosse = true
                    topCardIsMyCard = true
                    topCardFace = topCardFace.next
                    bottomCardFace = bottomCardFace.next
                    titleId = R.string.game_title_choosePower
                }

            }

            is GameScreenEvent.OnBottomCardClick -> {
                if (!hasChosse) {
                    hasChosse = true
                    bottomCardIsMyCard = true
                    topCardFace = topCardFace.next
                    bottomCardFace = bottomCardFace.next
                    titleId = R.string.game_title_choosePower
                }

            }

            is GameScreenEvent.OnPowerButtonClick -> {
                chosenPower = event.power
                chosenPowerValueTopCard =
                    parsePowerValue(hero = heroTopCard, powerName = chosenPower)
                chosenPowerValueBottomCard =
                    parsePowerValue(hero = heroBottomCard, powerName = chosenPower)
                startBetAnimation = true
                when (event.card) {
                    CardPosition.TOP -> {
                        viewModelScope.launch {
                            delay(3000)
                            if (chosenPowerValueTopCard > chosenPowerValueBottomCard) {
                                youHaveWon = true
                            } else if (chosenPowerValueTopCard < chosenPowerValueBottomCard) {
                                youHaveLost = true
                            } else {
                                youHaveTied = true
                            }
                        }
                    }

                    CardPosition.BOTTOM -> {
                        viewModelScope.launch {
                            delay(3000)
                            if (chosenPowerValueBottomCard > chosenPowerValueTopCard) {
                                youHaveWon = true
                            } else if (chosenPowerValueBottomCard < chosenPowerValueTopCard) {
                                youHaveLost = true
                            } else {
                                youHaveTied = true
                            }
                        }

                    }
                }
            }

            is GameScreenEvent.OnRestartClick -> {
                reStart()
            }

            is GameScreenEvent.OnExitClick -> {
                youHaveLost = false
                youHaveWon = false
                youHaveTied = false
            }
        }
    }

    //Reiniciem el joc, tornem a posar les cartes boca avall i agafem nous herois
    private fun reStart() {
        topCardIsMyCard = false
        bottomCardIsMyCard = false
        topCardFace = CardFace.Front
        bottomCardFace = CardFace.Front
        titleId = R.string.game_title_chooseCard
        startBetAnimation = false
        youHaveLost = false
        youHaveWon = false
        youHaveTied = false
        viewModelScope.launch {
            delay(1000)
            getCards()
            hasChosse = false
        }
    }

}