package com.mmoragues.herois.ui.screens.gameScreen.flipCard

import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.Button
import androidx.compose.material.ButtonDefaults
import androidx.compose.material.MaterialTheme
import androidx.compose.material.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.clip
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.testTag
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.semantics.contentDescription
import androidx.compose.ui.semantics.semantics
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import coil.compose.rememberImagePainter
import com.mmoragues.herois.R
import com.mmoragues.herois.common.TestConstants.BETBOTTOMS
import com.mmoragues.herois.domain.model.Hero
import com.mmoragues.herois.ui.screens.gameScreen.GameScreenEvent
import com.mmoragues.herois.ui.sharedComponents.powerStat.PowerName
import com.mmoragues.herois.ui.sharedComponents.powerStat.PowerStatBar
import com.mmoragues.herois.ui.sharedComponents.powerStat.parsePowerColor
import com.mmoragues.herois.ui.sharedComponents.powerStat.parsePowerName
import com.mmoragues.herois.ui.theme.*


/**
 * Cara posterior de la carta
 * Segons si és la carta del judador o no es mostraran coses diferents
 */
@Composable
fun BackCard(
    navController: NavController,
    modifier: Modifier = Modifier,
    hero: Hero,
    position: CardPosition,
    onEvent: (GameScreenEvent) -> Unit,
    isMyCard: Boolean,
    startBetAnimation: Boolean,
    chosenPower: PowerName,
    chosenPowerValue: Int,
    youHaveWon: Boolean,
    youHaveLost: Boolean,
    youHaveTied: Boolean
) {

    Row(
        modifier = modifier
            .semantics { contentDescription = "BackCard" }
            .fillMaxSize()
            .padding(10.dp)
    ) {

        //Mostrem la foto i el nom del heroi en una columna a l'esquerra
        Column(
            modifier = Modifier
                .weight(0.3f)
                .align(Alignment.CenterVertically)
                .padding(5.dp),
            verticalArrangement = Arrangement.Center,
            horizontalAlignment = Alignment.Start
        ) {

            val painter = rememberImagePainter(
                data = hero.images.sm,
                builder = {
                    error(R.drawable.offline)
                    crossfade(500)
                }
            )

            Image(
                modifier = Modifier
                    .size(150.dp, 150.dp)
                    .clip(RoundedCornerShape(16.dp)),
                painter = painter,
                alignment = Alignment.Center,
                contentDescription = hero.name,
                contentScale = ContentScale.Crop
            )

            Text(
                text = hero.name,
                textAlign = TextAlign.Center,
                modifier = Modifier.align(Alignment.CenterHorizontally),
                style = MaterialTheme.typography.body1,
                fontWeight = FontWeight.Bold
            )
        }

        //Si es la meva carta i encara no he apostat, mostrem els botons per apostar
        if (isMyCard && !startBetAnimation) {
            BetBottoms(
                modifier = Modifier
                    .weight(0.35f)
                    .testTag(BETBOTTOMS),
                position = position,
                onEvent = onEvent
            )
        }

        //Si no es la meva carta i encara no s'ha apostat, mostrem un missatge
        if (!isMyCard && !startBetAnimation) {
            Column(
                Modifier
                    .weight(0.7f)
                    .align(Alignment.CenterVertically),
                horizontalAlignment = Alignment.CenterHorizontally,
                verticalArrangement = Arrangement.Center
            ) {
                Text(text = stringResource(id = R.string.card_choose_power))
            }
        }

        //Si ja s'ha fet l'aposta, mostrem la barra animada amb el poder elegit
        if (startBetAnimation) {
            Column(
                Modifier
                    .weight(0.7f)
                    .align(Alignment.CenterVertically),
                horizontalAlignment = Alignment.CenterHorizontally,
                verticalArrangement = Arrangement.Center
            ) {
                PowerStatBar(
                    powerNameId = parsePowerName(chosenPower),
                    powerValue = chosenPowerValue,
                    powerColor = parsePowerColor(chosenPower),
                    animDuration = 2000
                )

            }
        }

        //Si és la meva carta, mostrem els diàlegs amb el resultat
        if (isMyCard) {
            //Has guanyat
            if (youHaveWon) {
                ResultDialog(
                    navController = navController,
                    onEvent = onEvent,
                    title = stringResource(id = R.string.result_title_win),
                    desc = stringResource(id = R.string.result_desc_win),
                    rawRes = R.raw.gamewinner,
                    color = heroGreenColor
                )
            }

            //Has pedut
            if (youHaveLost) {
                ResultDialog(
                    navController = navController,
                    onEvent = onEvent,
                    title = stringResource(id = R.string.result_title_lose),
                    desc = stringResource(id = R.string.result_desc_lose),
                    rawRes = R.raw.tryagain,
                    color = heroRedColor
                )
            }

            //És un empat
            if (youHaveTied) {
                ResultDialog(
                    navController = navController,
                    onEvent = onEvent,
                    title = stringResource(id = R.string.result_title_tie),
                    desc = stringResource(id = R.string.result_desc_tie),
                    rawRes = R.raw.tie,
                    color = heroOrangeColor
                )
            }
        }
    }
}

/**
 * Botoms amb les opcions que pot apostar el jugador
 */
@Composable
fun BetBottoms(
    modifier: Modifier = Modifier,
    position: CardPosition,
    onEvent: (GameScreenEvent) -> Unit
) {

    //Primera columna de botons
    Column(
        modifier = modifier
            .fillMaxHeight()
            .padding(4.dp),
        verticalArrangement = Arrangement.SpaceEvenly,
    ) {
        //Botó Combat
        Button(
            modifier = Modifier
                .fillMaxWidth(),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = combatColor,
                contentColor = Color.Black
            ),
            onClick = {
                onEvent(
                    GameScreenEvent.OnPowerButtonClick(
                        position,
                        PowerName.COMBAT
                    )
                )
            },
            contentPadding = PaddingValues(0.dp)
        ) {
            Text(
                text = stringResource(id = R.string.power_combat),
                modifier = Modifier.align(Alignment.CenterVertically)
            )
        }

        Spacer(modifier = Modifier.height(5.dp))

        //Botó Durabilitat
        Button(
            modifier = Modifier
                .fillMaxWidth(),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = durabilityColor,
                contentColor = Color.Black
            ),
            onClick = {
                onEvent(
                    GameScreenEvent.OnPowerButtonClick(
                        position,
                        PowerName.DURABILITY
                    )
                )
            },
            contentPadding = PaddingValues(0.dp)
        )
        {
            Text(
                text = stringResource(id = R.string.power_durability),
                modifier = Modifier.align(Alignment.CenterVertically)
            )
        }

        Spacer(modifier = Modifier.height(5.dp))

        //Botó Intel·ligència
        Button(
            modifier = Modifier
                .fillMaxWidth(),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = intelligenceColor,
                contentColor = Color.Black
            ),
            onClick = {
                onEvent(
                    GameScreenEvent.OnPowerButtonClick(
                        position,
                        PowerName.INTELLIGENCE
                    )
                )
            },
            contentPadding = PaddingValues(0.dp)
        ) {
            Text(
                text = stringResource(id = R.string.power_intelligence),
                modifier = Modifier.align(Alignment.CenterVertically)
            )
        }
    }

    //Segona columna de botons
    Column(
        modifier = modifier
            .fillMaxHeight()
            .padding(4.dp),
        verticalArrangement = Arrangement.SpaceEvenly

    ) {
        //Botó Poder
        Button(
            modifier = Modifier
                .fillMaxWidth(),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = powerColor,
                contentColor = Color.Black
            ),
            onClick = {
                onEvent(
                    GameScreenEvent.OnPowerButtonClick(
                        position,
                        PowerName.POWER
                    )
                )
            },
            contentPadding = PaddingValues(0.dp)
        ) {
            Text(
                text = stringResource(id = R.string.power_power),
                modifier = Modifier.align(Alignment.CenterVertically)
            )
        }


        Spacer(modifier = Modifier.height(5.dp))

        //Boto Velocitat
        Button(
            modifier = Modifier
                .fillMaxWidth(),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = speedColor,
                contentColor = Color.Black
            ),
            onClick = {
                onEvent(
                    GameScreenEvent.OnPowerButtonClick(
                        position,
                        PowerName.SPEED
                    )
                )
            },
            contentPadding = PaddingValues(0.dp)
        ) {
            Text(
                text = stringResource(id = R.string.power_speed),
                modifier = Modifier.align(Alignment.CenterVertically)
            )
        }

        Spacer(modifier = Modifier.height(5.dp))

        //Botó Força
        Button(
            modifier = Modifier
                .fillMaxWidth(),
            colors = ButtonDefaults.buttonColors(
                backgroundColor = strengthColor,
                contentColor = Color.Black
            ),
            onClick = {
                onEvent(
                    GameScreenEvent.OnPowerButtonClick(
                        position,
                        PowerName.STRENGTH
                    )
                )
            },
            contentPadding = PaddingValues(0.dp)
        ) {
            Text(
                text = stringResource(id = R.string.power_strength),
                modifier = Modifier.align(Alignment.CenterVertically)
            )
        }
    }

}