package com.mmoragues.herois.ui.screens.gameScreen.flipCard

import androidx.compose.animation.core.FastOutSlowInEasing
import androidx.compose.animation.core.animateFloatAsState
import androidx.compose.animation.core.tween
import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.Card
import androidx.compose.runtime.Composable
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.graphicsLayer
import androidx.compose.ui.platform.testTag
import androidx.compose.ui.unit.dp

/**
 * Cares de la carta (frontal i trasera)
 */
enum class CardFace(val angle: Float) {
    Front(0f) {
        override val next: CardFace
            get() = Back
    },
    Back(180f) {
        override val next: CardFace
            get() = Front
    };

    abstract val next: CardFace
}

/**
 * Possibles rotacions que pot fer la carta, sobre el eix X o sobre el eix Y
 */
enum class RotationAxis {
    AxisX,
    AxisY,
}

/**
 * Composable d'una carta que es pot donar la volta
 * Es dona la volta en 1,5 segons
 * Pot rotar en el eix X o Y
 */
@Composable
fun FlipCard(
    cardFace: CardFace,
    onClick: () -> Unit,
    modifier: Modifier = Modifier.testTag("FlipCard"),
    axis: RotationAxis = RotationAxis.AxisX,
    back: @Composable () -> Unit = {},
    front: @Composable () -> Unit = {},
) {
    val rotation = animateFloatAsState(
        targetValue = cardFace.angle,
        animationSpec = tween(
            durationMillis = 1500,
            easing = FastOutSlowInEasing,
        )
    )

    Card(
        modifier = modifier
            .testTag("Card")
            .padding(8.dp)
            .fillMaxWidth()
            .height(250.dp)
            .graphicsLayer {
                if (axis == RotationAxis.AxisX) {
                    rotationX = rotation.value
                } else {
                    rotationY = rotation.value
                }
                cameraDistance = 12f * density
            }
            .clickable { onClick() },
        elevation = 16.dp,
        shape = RoundedCornerShape(20.dp),
    ) {
        Box(
            modifier = modifier
                .testTag("Box Front")
                .fillMaxSize()
        ) {
            if (rotation.value <= 90f) {
                front()
            } else {

                Box(
                    modifier
                        .testTag("Box Back")
                        .fillMaxSize()
                        .graphicsLayer {
                            if (axis == RotationAxis.AxisX) {
                                rotationX = 180f
                            } else {
                                rotationY = 180f
                            }
                        },
                ) {
                    back()
                }
            }
        }
    }
}