package com.mmoragues.herois.ui.screens.homeScreen

import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.*
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.platform.testTag
import androidx.compose.ui.res.stringResource
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavHostController
import com.mmoragues.herois.R
import com.mmoragues.herois.common.TestConstants.LOADING
import com.mmoragues.herois.ui.navigation.Screen
import com.mmoragues.herois.ui.sharedComponents.BottomBar
import com.mmoragues.herois.ui.sharedComponents.ItemHeroCard
import com.mmoragues.herois.ui.sharedComponents.MainScaffold
import com.mmoragues.herois.ui.sharedComponents.searchBar.MainAppBar
import com.mmoragues.herois.ui.sharedComponents.searchBar.SearchWidgetState
import kotlinx.coroutines.flow.collect

/**
 * Pantall d'Inici de l'aplicació
 */
@Composable
fun HomeScreen(
    navController: NavHostController,
    viewModel: HomeViewModel = hiltViewModel()
) {
    val context = LocalContext.current

    // Variables d'estat de la pantalla
    val scaffoldState = rememberScaffoldState()
    val heroes = viewModel.heroes
    val searchWidgetState = viewModel.searchWidgetState
    val searchTextState = viewModel.searchTextState
    val noHeroesMessage = viewModel.noHeroesMessage
    val isUpdatingHeroes = viewModel.isUpdatingHeroes

    LaunchedEffect(key1 = scaffoldState) {
        viewModel.eventFlow.collect { event ->
            when (event) {
                is HomeScreenUIEvent.ShowSnackbar -> {
                    scaffoldState.snackbarHostState.showSnackbar(
                        message = context.getString(event.messageId),
                        duration = SnackbarDuration.Short
                    )
                }
            }
        }
    }

    /** Scaffold de la pantalla
     * Mostrem la barra superior amb el camp de text per buscar per nom
     * Mostrem el llistat de tots els herois
     * També la barra inferior per navegar entre les pantalles
     */
    MainScaffold(
        scaffoldState = scaffoldState,
        topBar = {
            MainAppBar(
                title = stringResource(id = R.string.homeScreen_title),
                searchWidgetState = searchWidgetState,
                searchTextState = searchTextState,
                onTextChange = {
                    viewModel.onEvent(HomeScreenEvent.OnUpdateSearchTextState(it))
                },
                onCloseClicked = {
                    viewModel.onEvent(
                        HomeScreenEvent.OnUpdateSearchWidgetState(
                            SearchWidgetState.CLOSED
                        )
                    )
                },
                onSearchClicked = {},
                onSearchTriggered = {
                    viewModel.onEvent(
                        HomeScreenEvent.OnUpdateSearchWidgetState(
                            SearchWidgetState.OPENED
                        )
                    )
                }
            )
        },
        bottomBar = { BottomBar(navController = navController) }
    ) { paddingValues ->

        //Si no hi ha herois a la llista, es mostra el missatge indicant el motiu
        // Pot ser perquè la búsqueda no té resultats o perquè no s'han carregat els herois
        if (noHeroesMessage != -1) {
            Column(
                modifier = Modifier.fillMaxSize(),
                verticalArrangement = Arrangement.Center,
                horizontalAlignment = Alignment.CenterHorizontally
            ) {
                Text(
                    text = stringResource(id = noHeroesMessage),
                    color = MaterialTheme.colors.error,
                    style = MaterialTheme.typography.h6
                )

            }
        }



        //Llistat amb els herois
        LazyColumn(
            modifier = Modifier.fillMaxSize(),
            contentPadding = paddingValues
        ) {
            items(heroes) { hero ->
                ItemHeroCard(
                    hero = hero,
                    onItemClicked = { hero ->
                        if (!isUpdatingHeroes){
                            navController.navigate(Screen.HeroDetailScreen.route + "/${hero.id}")
                        }
                    },
                    onFavoriteClicked = { hero ->
                        if (!isUpdatingHeroes){
                            viewModel.onEvent(
                                HomeScreenEvent.OnFavoriteClick(hero)
                            )
                        }
                    }
                )
            }
        }

        //Mentre està carregant les dades del servidor mostrem una indicador circular de progrés
        if (isUpdatingHeroes) {
            Column(
                modifier = Modifier.fillMaxSize()
                    .background(Color.White.copy(alpha = 0.5f))
                    .testTag(LOADING),
                verticalArrangement = Arrangement.Center,
                horizontalAlignment = Alignment.CenterHorizontally,

            ) {
                CircularProgressIndicator()
            }
        }
    }

}