package com.mmoragues.herois.ui.screens.homeScreen

import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.mmoragues.herois.R
import com.mmoragues.herois.common.Resource
import com.mmoragues.herois.domain.model.Hero
import com.mmoragues.herois.domain.useCase.GetAllHeroesUseCase
import com.mmoragues.herois.domain.useCase.ToggleFavoriteUseCase
import com.mmoragues.herois.ui.sharedComponents.searchBar.SearchWidgetState
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableSharedFlow
import kotlinx.coroutines.flow.asSharedFlow
import kotlinx.coroutines.flow.launchIn
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * View Model de la pantalla d'inici
 */
@HiltViewModel
class HomeViewModel @Inject constructor(
    private val getAllHeroesUseCase: GetAllHeroesUseCase,
    private val toggleFavoriteUseCase: ToggleFavoriteUseCase
) : ViewModel() {
    //Event per mostrar el Snackbar
    private val _eventFlow = MutableSharedFlow<HomeScreenUIEvent>()
    val eventFlow = _eventFlow.asSharedFlow()
    private var sentEvent: Boolean = false

    //Estat de widget per filtrar els herois (obert o tancat)
    var searchWidgetState by mutableStateOf(SearchWidgetState.CLOSED)
        private set

    //Valor del text a buscar
    var searchTextState by mutableStateOf("")
        private set

    //Llistat de Herois de la BBDD
    private var cachedHeroList = listOf<Hero>()

    //Llistat d'herois una vegada filtrats pel nom que surti al requadre de búsqueda
    var heroes by mutableStateOf(listOf<Hero>())
        private set

    //Missatge que es mostra si no hi ha herois a la llista
    var noHeroesMessage by mutableStateOf(-1)
        private set

    //Indica que està actualitzant els herois
    var isUpdatingHeroes by mutableStateOf(false)
        private set


    init {
        getHeroes()
    }

    private fun getHeroes() {
        getAllHeroesUseCase().onEach { result ->
            cachedHeroList = result.data ?: emptyList()
            when (result) {
                is Resource.Loading -> {
                    isUpdatingHeroes = true
                    heroes = cachedHeroList.filter { hero ->
                        hero.name.contains(
                            searchTextState,
                            ignoreCase = true
                        )
                    }
                }
                is Resource.Error -> {
                    isUpdatingHeroes = false
                    heroes = cachedHeroList.filter { hero ->
                        hero.name.contains(
                            searchTextState,
                            ignoreCase = true
                        )
                    }
                    //Enviem el missatge per mostrar el snackbar només una vegada
                    if (!sentEvent) {
                        sentEvent = true
                        _eventFlow.emit(
                            HomeScreenUIEvent.ShowSnackbar(
                                result.messageId ?: R.string.error_unexpected
                            )
                        )
                    }
                    updateNoHeroesMessage()

                }
                is Resource.Success -> {
                    isUpdatingHeroes = false
                    heroes = cachedHeroList.filter { hero ->
                        hero.name.contains(
                            searchTextState,
                            ignoreCase = true
                        )
                    }

                    updateNoHeroesMessage()
                }
            }

        }.launchIn(viewModelScope)
    }

    fun onEvent(event: HomeScreenEvent) {
        when (event) {
            is HomeScreenEvent.OnFavoriteClick -> {
                viewModelScope.launch {
                    toggleFavoriteUseCase(event.hero)
                }
            }
            is HomeScreenEvent.OnUpdateSearchWidgetState -> {
                searchWidgetState = event.newState
            }
            is HomeScreenEvent.OnUpdateSearchTextState -> {
                searchTextState = event.newValue
                heroes = cachedHeroList.filter { hero ->
                    hero.name.contains(
                        event.newValue,
                        ignoreCase = true
                    )
                }
                updateNoHeroesMessage()
            }
        }
    }

    private fun updateNoHeroesMessage() {
        //Actualitza el missatge a mostra en cas que no hi hagi herois a mostrar
        //Pot ser degut a que la búsqueda no dona resultats o perquè no s'han carregat els herois
        noHeroesMessage = when {
            cachedHeroList.isEmpty() -> {
                R.string.heroesList_empty
            }
            heroes.isEmpty() -> {
                R.string.heroesList_noResult
            }
            else -> {
                -1
            }
        }
    }

}