package com.mmoragues.herois.data.repository

import com.google.common.truth.Truth.assertThat
import com.mmoragues.herois.common.Resource
import com.mmoragues.herois.data.database.FakeLocalDataSource
import com.mmoragues.herois.data.remote.FakeRemoteDataSource
import com.mmoragues.herois.domain.model.Hero
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.runBlocking
import org.junit.Before
import org.junit.Test

// Proves del repository on es fa servir una Local Data Source i una Remote Data Source "Fake"
class HeroRepositoryImplTest {
    private lateinit var fakeLocalDataSource: FakeLocalDataSource
    private lateinit var fakeRemoteDataSource: FakeRemoteDataSource
    private lateinit var fakeRemoteDataSourceError: FakeRemoteDataSource
    private lateinit var fakeLocalHeroes: MutableList<Hero>
    private lateinit var fakeRemoteHeroes: MutableList<Hero>

    @Before
    fun setUp() {
        fakeLocalDataSource = FakeLocalDataSource()
        fakeRemoteDataSource = FakeRemoteDataSource()
        fakeRemoteDataSourceError = FakeRemoteDataSource(throwException = true)

        fakeLocalHeroes = mutableListOf()
        ('a'..'c').forEachIndexed { index, c ->
            fakeLocalHeroes.add(Hero(id = index, name = c.toString()))
        }
        fakeLocalDataSource.insertFakeHeroes(fakeLocalHeroes)

        fakeRemoteHeroes = mutableListOf()
        ('a'..'z').forEachIndexed { index, c ->
            fakeRemoteHeroes.add(Hero(id = index, name = c.toString()))
        }
        fakeRemoteDataSource.insertFakeHeroes(fakeRemoteHeroes)
    }

    // Comprovem que podem aptenir tots els herois. Rebem primer les dades de la cache i després una vegada que s'ha actualitzat
    @Test
    fun `Get All Heroes`() = runBlocking {

        val heroRepositoryImpl = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSource)

        heroRepositoryImpl.getAllHeroes().collect { result ->
            when (result) {
                is Resource.Loading -> {
                    assertThat(result.data).isEqualTo(fakeLocalHeroes)
                    System.out.println("Loading OK")
                }
                is Resource.Success -> {
                    assertThat(result.data).isEqualTo(fakeRemoteHeroes)
                    System.out.println("Success OK")
                }
                is Resource.Error -> {
                    System.out.println("No hauria d'haver rebut un Resource.Error")
                    assertThat(true).isFalse()

                }
            }
        }
    }

    // El servidor fake simula un error i comprovem que el respository ho gestiona correctament
    @Test
    fun `Get All Heroes with Server Error`() = runBlocking {
        val heroRepositoryImpl = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSourceError)

        heroRepositoryImpl.getAllHeroes().collect { result ->
            when (result) {
                is Resource.Loading -> {
                    assertThat(result.data).isEqualTo(fakeLocalHeroes)
                    System.out.println("Loading OK")
                }
                is Resource.Success -> {
                    System.out.println("No hauria d'haver rebut un Resource.Success")
                    assertThat(true).isFalse()
                }
                is Resource.Error -> {
                    assertThat(result.messageId).isNotNull()
                    assertThat(result.data).isEqualTo(fakeLocalHeroes)
                    System.out.println("Error OK")
                }
            }
        }
    }

   //Obtenim un heroi pel seu ID
    @Test
    fun `Get a Hero by its ID`() = runBlocking {

        val heroRepositoryImpl = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSource)
        heroRepositoryImpl.getAllHeroes().collect()

        val heroRecuperat = heroRepositoryImpl.getHeroById(0).first()

        assertThat(heroRecuperat).isEqualTo(fakeLocalHeroes[0])
        System.out.println("Get a Hero By ID OK")
    }

    // Provem que es pot posar un heroi com a favorit correctament
    @Test
    fun `Toggle is Favorite`() = runBlocking {

        val heroRepositoryImpl = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSource)

        heroRepositoryImpl.toggleIsFavorite(0)

        val heroRecuperat = heroRepositoryImpl.getHeroById(0).first()

        assertThat(heroRecuperat.isFavorite).isTrue()
        System.out.println("Toggle is Favorite OK")
    }

    // Provem d'optenir un heroi de aleatori
    @Test
    fun `Get a Random Hero`() = runBlocking {

        val heroRepositoryImpl = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSource)
        heroRepositoryImpl.getAllHeroes().collect()

        val randomHero = heroRepositoryImpl.getRandomHero()
        assertThat(randomHero).isIn(fakeRemoteHeroes)

        System.out.println("Random Hero OK - ID hero: ${randomHero.id}")
    }

    // Provem d'optenir la llista d'herois favorits
    @Test
    fun `Get Favorites Heroes`() = runBlocking {

        val heroRepositoryImpl = HeroRepositoryImpl(fakeLocalDataSource, fakeRemoteDataSource)

        heroRepositoryImpl.toggleIsFavorite(0)

        val favoritesHeroes = heroRepositoryImpl.getFavoritesHeroes().first()
        assertThat(favoritesHeroes).containsExactly(fakeLocalHeroes[0])
        System.out.println("Get Favorites Heroes OK")
    }
}